<?php

namespace Teknisa\DF\Modules\Entrada\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use \Zeedhi\Framework\DTO\Response\Error;
use Teknisa\DF\Modules\Entrada\Services\Dfe07900ControleDeArquivosXmlService;
use Zeedhi\Framework\DataSource\DataSet;

class Dfe07900ControleDeArquivosXmlController extends \Teknisa\DF\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $parameterBagColumns;
    protected $controleArquivosXmlService;
    
    public function __construct($crudService, $parameterBagColumns, Dfe07900ControleDeArquivosXmlService $controleArquivosService, $environment, $fileServerBaseUrl) {
        $this->crudService             = $crudService;
        $this->parameterBagColumns     = $parameterBagColumns;
        $this->controleArquivosService = $controleArquivosService; 
        $this->isSaas            = $environment->getNrOrgTrab() != 1;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
    }
    
    public function find(Request\Filter $request, Response $response) {
        
        $conditions = $request->getFilterCriteria()->getConditions();
        if($this->isSaas) {
            if(isset($conditions[0])) {
                $dir  = $conditions[0]["value"];
                $rows = $this->controleArquivosService->getArquivosXMLTabela($dir);
            }
            else {
                $rows = $this->controleArquivosService->getBaseDirTabela();
            }
        }
        else {
            if(isset($conditions[0])) {
                $dir  = $conditions[0]["value"];
                $rows = $this->controleArquivosService->getArquivosXMLDiretorio($dir, false);
            }
            else {
                $rows = $this->controleArquivosService->getDirRaiz();
                if($rows == "Diretorio nao definido"){
                    $response->setError(new Error("Não há diretório parametrizado. Favor indicar o diretório onde estão os arquivos a serem importados na tela Parâmetros de Documentos Fiscais de Entrada.", 500));
                } else if($rows == "Diretorio nao encontrado"){
                    $response->setError(new Error("O diretório parametrizado não foi encontrado.", 500));
                }
            }
            
        }

        if(!$response->getError()){
            $dataSet = new DataSet("arquivos_xml", $rows);
            $response->setDataSets(array($dataSet));
        } else{
            $response->setStatus(500);
        }
    }
    
    public function getIsSaas(Request\Filter $request, Response $response){
        $dataSet = new DataSet("isSaas", [$this->isSaas]);
        $response->setDataSets(array($dataSet));
    }
    
    /*
    * Cria diretório para arquivos XML
    **/
    public function novoDirArqXML(Request\Row $request, Response $response){
        
    }
    
    /*
    * Insere o arquivo XML no diretório 
    **/
    public function uploadArqXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->controleArquivosService->uploadArqXML($row["DIR"], $row["ARQUIVO"], $this->isSaas);
        
    }
    
    /*
    * Realiza o download do arquivo XML
    **/
    public function downloadArqXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        if($this->isSaas) {
            $this->controleArquivosService->downloadArqXML($row["DSARQUIVO"], $this->fileServerBaseUrl);
        }
        else {
            $this->controleArquivosService->downloadArqXML($row["DSARQUIVO"]);
        }
    }
    
    /*
    * Exclui o arquivo XML do diretório
    **/
    public function excluirArqXML(Request\Row $request, Response $response){
        
    }
    
}