<?php
namespace Teknisa\DF\Modules\Entrada\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Response;
use Teknisa\DF\Modules\Entrada\Services\Dfe43000ParametrosPorFornecedorService;
use Teknisa\DF\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

class Dfe43000ParametrosPorFornecedorController extends TeknisaCrud {
    
    protected $service;
    protected $environment;
    protected $dataSourceName = "paramforne_fornecedor";
    
    public function __construct($crudService, $parameterBag, $service, $environment){
        parent::__construct($crudService, $parameterBag);
        $this->service = $service;
        $this->environment = $environment;
    }
    
    public function findFornecedor(DTO\Request\Filter $request, DTO\Response $response) {
        $filialFranqueadora = $this->service->getFranqueadora();
        if($filialFranqueadora) {
            $franqueadora = ($filialFranqueadora == $this->environment->getCdfilial()) ? 'S' : 'N';
            $conditions = $request->getFilterCriteria()->getConditions();
            $request->getFilterCriteria()->addCondition("P_CDFILIAL",     "=", $this->environment->getCdfilial());
            $request->getFilterCriteria()->addCondition("P_FRANQUEADORA", "=", $franqueadora);
            $this->dataSourceName = 'paramforne_fornecedor_filial';
            parent::find($request, $response);
        }
        else {
            $this->dataSourceName = 'paramforne_fornecedor';
            parent::find($request, $response);
        }
    } 
    
    public function saveCadastroAutomatico($request, $response){
        $rowsNovas = (array)$request->getRow();
        $array = array();
        $cdAlterados = $rowsNovas["CDFORNECED"];
        
        $condition = $this->retornaCondicao($rowsNovas);
        $rowsAntigas = $this->getRowsAntigas($condition);
        
        unset($rowsNovas["CDFORNECED"]);
        
        foreach($rowsAntigas->getRows() as $row){
            foreach($rowsNovas as $rowKey => $rowValue){
                $row[$rowKey] = $rowValue;
            }
            
            $array[] = $row;
        }
        try{
            $dataSet = new DataSet('paramforne_fornecedor', $array);
            $requestSalvar = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $saveRows = $this->crudService->getRowsToSave($requestSalvar, $this->dataSourceName);
            $this->crudService->save('paramforne_fornecedor', $saveRows);
        }catch (\Exception $e) {
        $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
        
    }
    
    public function retornaCondicao($row){
        $conditions = array();
        $conditions[] =
          array(
              "value" => $row['CDFORNECED'],
              "operator" => "IN",
              "columnName" => "CDFORNECED"
            );
        return $conditions;
    }
    
    public function getRowsAntigas($condition){
        $filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->service->buscar($newFilterCriteria);
        return $dataSet;
    }
}