<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;

class NFSeRioDeJaneiro {
    const URL_PRODUCAO = "https://notacarioca.rio.gov.br/WSNacional/nfse.asmx?wsdl";
    const URL_HOMOLOGACAO = "https://homologacao.notacarioca.rio.gov.br/WSNacional/nfse.asmx?wsdl";

    protected $connection;
    protected $environment;
    protected $newCode;

    protected $ambiente; //1 = producao, 2 = homologacao

    // Dados para identificar o Certificado Digital
    protected $cdempresa;
    protected $cdinscmuni;

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $xmlConsultarLoteRps;

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoXml;

    protected $senhaCertificadoDigital;

    // Dados Prestador
    protected $InscricaoPrestador;
    protected $InscricaoMunicipalPrestador;
    protected $RazaoSocialPrestador;
    protected $CodigoMunicipioPrestador;

    // Dados Lote RPS
    protected $IdLote;
	protected $NumeroLote;

	// Dados RPS
    protected $IdRps;
	protected $NumeroRps;
	protected $SerieRps;
	protected $TipoRps;
	protected $DataEmissao;
	protected $NaturezaOperacao;
	protected $RegimeEspecialTributacao;
	protected $OptanteSimplesNacional;
	protected $IncentivadorCultural;
	protected $StatusRps;
	protected $ValorServicos;
	protected $ValorDeducoes;
	protected $ValorPis;
	protected $ValorCofins;
	protected $ValorInss;
	protected $ValorIr;
	protected $ValorCsll;
	protected $IssRetido;
	protected $ValorIss;
	protected $ValorIssRetido;
	protected $OutrasRetencoes;
	protected $BaseCalculo;
	protected $Aliquota;
	protected $ValorLiquidoNfse;
	protected $DescontoCondicionado;
	protected $DescontoIncondicionado;
	protected $ItemListaServico;
	protected $CodigoCnae;
	protected $CodigoTributacaoMunicipio;
	protected $Discriminacao;
	protected $CodigoMunicipioTomador;
	protected $InscricaoTomador;
	protected $InscricaoMunicipalTomador;
	protected $RazaoSocialTomador;
	protected $Endereco;
	protected $Numero;
	protected $Complemento;
	protected $Bairro;
	protected $Uf;
	protected $Cep;
	protected $Telefone;
	protected $Email;

	public function __construct($connection, Environment $environment, $newCode, $xmlDir, $certUploadDir = null, $certUploadLink = null, $certKeyLink = null) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;

        //$this->setCaminhoXml($xmlDir);
        if($certUploadDir) {
            $this->setCaminhoCertificado($certUploadDir);
        }
        else {
            $this->linkCertificado = $certUploadLink;
            $this->linkCertKey     = $certKeyLink;
        }
	}

	private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}

	private function gerarXmlEnviarLoteRps() {
		$this->xmlEnviarLoteRps = '
		<EnviarLoteRpsEnvio xmlns="http://www.abrasf.org.br/ABRASF/arquivos/nfse.xsd">
			<LoteRps Id="'.$this->IdLote.'">
				<NumeroLote>'.$this->NumeroLote.'</NumeroLote>
				<Cnpj>'.$this->InscricaoPrestador.'</Cnpj>
				<InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
				<QuantidadeRps>'.count($this->listaRps).'</QuantidadeRps>
				<ListaRps>';

				foreach ($this->listaRps as $rps) {
                    $this->xmlEnviarLoteRps .= "\n" . $rps;
                }

                $this->xmlEnviarLoteRps .= '
                </ListaRps>
			</LoteRps>
		</EnviarLoteRpsEnvio>';
	}

	private function adicionarRpsEnviarLote() {
		$xmlRps = '
				<Rps>
					<InfRps Id="'.$this->IdRps.'">
						<IdentificacaoRps>
							<Numero>'.$this->NumeroRps.'</Numero>
							<Serie>'.$this->SerieRps.'</Serie>
							<Tipo>'.$this->TipoRps.'</Tipo>
						</IdentificacaoRps>
						<DataEmissao>'.$this->DataEmissao.'</DataEmissao>
						<NaturezaOperacao>'.$this->NaturezaOperacao.'</NaturezaOperacao>
						<OptanteSimplesNacional>'.$this->OptanteSimplesNacional.'</OptanteSimplesNacional>
						<IncentivadorCultural>'.$this->IncentivadorCultural.'</IncentivadorCultural>
						<Status>'.$this->StatusRps.'</Status>
						<Servico>
							<Valores>
								<ValorServicos>'.Util::formatarNumero($this->ValorServicos).'</ValorServicos>
								<ValorDeducoes>'.Util::formatarNumero($this->ValorDeducoes).'</ValorDeducoes>
								<ValorPis>'.Util::formatarNumero($this->ValorPis).'</ValorPis>
								<ValorCofins>'.Util::formatarNumero($this->ValorCofins).'</ValorCofins>
								<ValorInss>'.Util::formatarNumero($this->ValorInss).'</ValorInss>
								<ValorIr>'.Util::formatarNumero($this->ValorIr).'</ValorIr>
								<ValorCsll>'.Util::formatarNumero($this->ValorCsll).'</ValorCsll>
								<IssRetido>'.$this->IssRetido.'</IssRetido>
								<ValorIss>'.Util::formatarNumero($this->ValorIss).'</ValorIss>
								<OutrasRetencoes>'.Util::formatarNumero($this->OutrasRetencoes).'</OutrasRetencoes>
								<Aliquota>'.$this->Aliquota.'</Aliquota>
								<DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</DescontoIncondicionado>
								<DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</DescontoCondicionado>
							</Valores>
							<ItemListaServico>'.$this->ItemListaServico.'</ItemListaServico>
							<CodigoTributacaoMunicipio>'.$this->CodigoTributacaoMunicipio.'</CodigoTributacaoMunicipio>
							<Discriminacao>'.$this->Discriminacao.'</Discriminacao>
							<CodigoMunicipio>'.$this->CodigoMunicipioPrestador.'</CodigoMunicipio>
						</Servico>
						<Prestador>
							<Cnpj>'.$this->InscricaoPrestador.'</Cnpj>
							<InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
						</Prestador>
						<Tomador>
							<IdentificacaoTomador>
								<CpfCnpj>
									';

								if (strlen($this->InscricaoTomador) == 11) {
									$xmlRps .= '<Cpf>'.$this->InscricaoTomador.'</Cpf>';
								}
								else {
									$xmlRps .= '<Cnpj>'.$this->InscricaoTomador.'</Cnpj>';
								}

								$xmlRps .= '
								</CpfCnpj>
							<InscricaoMunicipal>'.$this->InscricaoMunicipalTomador.'</InscricaoMunicipal>
							</IdentificacaoTomador>
							<RazaoSocial>'.$this->RazaoSocialTomador.'</RazaoSocial>
							<Endereco>
								<Endereco>'.$this->Endereco.'</Endereco>
								<Numero>'.$this->Numero.'</Numero>
								<Complemento>'.$this->Complemento.'</Complemento>
								<Bairro>'.$this->Bairro.'</Bairro>
								<CodigoMunicipio>'.$this->CodigoMunicipioTomador.'</CodigoMunicipio>
								<Uf>'.$this->Uf.'</Uf>
								<Cep>'.$this->Cep.'</Cep>
							</Endereco>
							<Contato>
							    <Email>'.$this->Email.'</Email>
							</Contato>
						</Tomador>
					</InfRps>
				</Rps>';

        /*
        valores

        <ValorIssRetido>'.Util::formatarNumero($this->ValorIssRetido).'</ValorIssRetido>
        <DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</DescontoCondicionado>
        <DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</DescontoIncondicionado>
        */

        $this->listaRps[] = $xmlRps;
	}

	private function getXmlAssinadoEnviarLoteRps() {
	    $this->xmlEnviarLoteRps = str_replace("&", "&#38;", $this->xmlEnviarLoteRps);
		$domXml = new \DOMDocument("1.0", "UTF-8");
		$domXml->loadXML($this->xmlEnviarLoteRps, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("InfRps", "LoteRps"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->limparListaRps();
            return $domXml->saveXML($domXml->documentElement);
        }
	}

    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes()) {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "RecepcionarLoteRps";

        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        if ($resultado->outputXML) {
            $nomeArquivoResposta = $prefixoArquivo."resp.xml";
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);

            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado->outputXML);

            if ($domResult->getElementsByTagName("NumeroLote")->length > 0) {

                $protocolo = $domResult->getElementsByTagName("Protocolo")->item(0)->nodeValue;

                foreach ($this->listaAuxRps as $nota) {
                    Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                }

                Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->NumeroLote, 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());

                return true;
            }
            else if ($domResult->getElementsByTagName("MensagemRetorno")->length > 0) {
                $erros = array("TIPO" => 2);
                $erros["XML"] = $xml;

                foreach ($domResult->getElementsByTagName("MensagemRetorno") as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                    );
                }
                
                file_put_contents("/tmp/".$nomeArquivoEnvio, $xml);
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarLoteRps";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }

    public function consultarLoteRps($lote) {
        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);

        $this->ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        //$this->RazaoSocialPrestador = $lote["NMRAZSOCEMPR"];
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
        $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $lote["CDINSCMUNI"]);
        $xml = $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);
        $resultado = $this->consultarLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);
        
        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            if ($domResult->getElementsByTagName("ListaNfse")->length > 0) {

                $nodeList = $domResult->getElementsByTagName("InfNfse");
                
                if ($nodeList->length > 0) {
                    foreach ($nodeList as $nota) {
                        $numeroNota = Util::getChildNodeByName($nota, "Numero")->nodeValue;
                        $codigoVerificacao = Util::getChildNodeByName($nota, "CodigoVerificacao")->nodeValue;
                        $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                        $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Serie")->nodeValue;
                        $numeroRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Numero")->nodeValue;
                        $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
    
                        /*==================================*/
                        $dataEmissao = Util::getChildNodeByName($nota, "DataEmissao")->nodeValue;
                        $dataEmissao = date_create_from_format("Y-m-d\TH:i:s", $dataEmissao);
                        $dataEmissao = $dataEmissao->format("d/m/Y");
    
                        $competencia = $nota->getElementsByTagName("Competencia")->item(0)->nodeValue;
                        $competencia = date_create_from_format("Y-m-d\TH:i:s", $competencia);
                        $competencia = $competencia->format("d/m/Y");
                        /*==================================*/
    
                        Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissao, $competencia, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                        Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                        Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    }
    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    return true;
                }
                else {
                    return null;
                }
            }
            else {
                $erros = array();
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                foreach ($nodeList as $erro) {
                    $codigoErro = Util::getChildNodeByName($erro, "Codigo")->nodeValue;
                    $descricaoErro = Util::getChildNodeByName($erro, "Mensagem")->nodeValue;
                    $nodeIdentificacaoRps = $erro->getElementsByTagName("IdentificacaoRps")->item(0);
                    $numeroRps = '';
                    if ($nodeIdentificacaoRps) {
                        $numeroRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Numero")->nodeValue;
                    }
                    
                    $erros[] = array(
                        "CODIGO" => $codigoErro,
                        "DESCRICAO" => $descricaoErro,
                        "RPS"    => $numeroRps
                    );
                    
                    // E92 e E180 = lote ainda nao foi processado
                    if ($codigoErro == "E92" || $codigoErro == "E180" || $codigoErro == "92" || $codigoErro == "180") {
                        return $erros;
                    }
                }

                $nota = $domResult->getElementsByTagName("MensagemRetorno")[0];
                $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                if($nodeIdentificacaoRps) {
                    $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Serie")->nodeValue;
                }
                else {
                    $serieRps = $lote["CDSERIERPS"];
                }

                Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $serieRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());

                Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());

                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function gerarXmlCancelarNota($nrnotafisc, $inscricaoJuridica, $inscricaoMunicipal, $codMotivoCancelamento, $cdmunicibge) {
        $this->xmlCancelarNota = '<CancelarNfseEnvio xmlns="http://www.abrasf.org.br/ABRASF/arquivos/nfse.xsd">
            <Pedido xmlns="http://www.abrasf.org.br/nfse.xsd">
                <InfPedidoCancelamento
                    Id="NFSE_'.$nrnotafisc.'">
                    <IdentificacaoNfse>
                        <Numero>'.$nrnotafisc.'</Numero>
                        <Cnpj>'.$inscricaoJuridica.'</Cnpj>
                        <InscricaoMunicipal>'.$inscricaoMunicipal.'</InscricaoMunicipal>
                        <CodigoMunicipio>'.$cdmunicibge.'</CodigoMunicipio>
                    </IdentificacaoNfse>
                    <CodigoCancelamento>'.$codMotivoCancelamento.'</CodigoCancelamento>
                </InfPedidoCancelamento>
            </Pedido>
        </CancelarNfseEnvio>';
    }

    private function getXmlAssinadoCancelarNota() {
        $this->xmlCancelarNota = str_replace("&", "&#38;", $this->xmlCancelarNota);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlCancelarNota, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("InfPedidoCancelamento"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            return $domXml->saveXML($domXml->documentElement);
        }
    }

    public function cancelarNotas($notas, $codMotivo, $dsMotivo, $cdmunicibge, $cdmunicsiafi) {
        $this->cdempresa = $notas[0]["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notas[0]["CDINSCMUNI"]);

        $this->ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());

        //$this->RazaoSocialPrestador = $notas[0]["NMRAZSOCEMPR"];
        $errosTotais = array();

        foreach ($notas as $nota) {
            $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $nota["NRINSJURIEST"]);
            $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $nota["CDINSCMUNI"]);
            $this->gerarXmlCancelarNota($nota["NRNOTAFISC"], $inscricaoJuridica, $inscricaoMunicipal, $codMotivo, $cdmunicibge);
            $xml = $this->getXmlAssinadoCancelarNota();

            $resultado = $this->cancelarNotaWS($xml, $nota["CDEMPRESA"], $nota["NRINSJURIEST"], $nota["NRNOTAFISC"]);

            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                if ($nodeList->length > 0) {
                    $errosNota = array();
                    foreach ($nodeList as $erro) {
                        $errosNota[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                        );
                    }
                    $errosTotais[$nota["NRLANCTONF"]] = $errosNota;
                }
                else {
                    $motivos = array(
                        1 => "Erro na emissão",
                        2 => "Serviço não prestado",
                        3 => "Erro na assinatura", 
                        4 => "Duplicidade da nota",
                        5 => "Erro no Processamento"
                    );
                    
                    $descMotivo = strtr($codMotivo, $motivos);

                    Util::inserirNotaCancelada($this->connection, "S", "C", $descMotivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
            }
            else {
                return false;
            }
        }
        return (count($errosTotais) > 0) ? $errosTotais : true;
    }

    private function cancelarNotaWS($xml, $cdempresa, $inscricao, $nrnotafisc) {
        $nomeFuncao = "CancelarNfse";
        $prefixoArquivo = $cdempresa.$inscricao.$nrnotafisc."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";

        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }

    private function enviarParaWebservice($nomeFuncao, $conteudo) {
        if($this->caminhoCertificado) {
            $caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni . DIRECTORY_SEPARATOR . "certKey.pem";
            $unlink  = false;
        }
        else {
            $certKey = $this->linkCertKey;
            $hash    = uniqid();
            if (is_dir("/tmp")) {
                $caminho= "/tmp/certKey$hash.pem";
            }
            else {
                $caminho = __DIR__ . DIRECTORY_SEPARATOR ."certKey$hash.pem";
            }
            file_put_contents($caminho, file_get_contents($certKey));
            $unlink = true;
        }

        $context = stream_context_create(array(
            'ssl' => array(
                'verify_peer' => false,
                'verify_peer_name' => false,
                'allow_self_signed' => true
            )
        ));

        $options = array(
            'local_cert' => $caminho,
            'trace' => 1,
            'soap_version' => SOAP_1_1,
            'exceptions' => true
        );
        $url = ($this->ambiente == 1) ? self::URL_PRODUCAO : self::URL_HOMOLOGACAO;
        
        $conteudo = str_replace("<", "&lt;", $conteudo);
        $conteudo = str_replace(">", "&gt;", $conteudo);
        
        $conteudo  ="<".$nomeFuncao.'Request xmlns="http://notacarioca.rio.gov.br/"><inputXML>'.$conteudo;
        
        $conteudo.="</inputXML></".$nomeFuncao."Request>";
        $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
        
        $client = new \SoapClient($url, $options);
        $resultado = $client->$nomeFuncao($soapVar);

        if($unlink) unlink($caminho);
        return $resultado;
    }

	private function assinaXml(\DOMDocument $domXml, array $tags) {
	    $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            //file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $domXml = $this->assinaXmlAux($domXml, $tag);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }

	private function assinaXmlAux(\DOMDocument $domXml, $tag) {
		$nodes = $domXml->getElementsByTagName($tag);
		foreach ($nodes as $node) {
			$root = $node->parentNode;
			$id = trim($node->getAttribute("Id"));
			$dados = $node->C14N(false, false, null, null);
			$hashValue = hash("sha1", $dados, true);
			$digValue = base64_encode($hashValue);

            $signature = $domXml->createElement("Signature");
            $root->appendChild($signature);
            $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
            $signature->setAttribute("Id", "Ass_".$id);

            $signedInfo = $domXml->createElement("SignedInfo");
	        $signature->appendChild($signedInfo);

	        $novaTag = $domXml->createElement("CanonicalizationMethod");
	        $signedInfo->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

	        $novaTag = $domXml->createElement("SignatureMethod");
	        $signedInfo->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");

	        $reference = $domXml->createElement("Reference");
	        $signedInfo->appendChild($reference);
	        $reference->setAttribute("URI", "#".$id);

	        $transforms = $domXml->createElement("Transforms");
	        $reference->appendChild($transforms);

	        $novaTag = $domXml->createElement("Transform");
	        $transforms->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

	        $novaTag = $domXml->createElement("Transform");
	        $transforms->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

	        $novaTag = $domXml->createElement("DigestMethod");
	        $reference->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");

	        $novaTag = $domXml->createElement("DigestValue", $digValue);
	        $reference->appendChild($novaTag);

            $dados = $signedInfo->C14N(false, false, null, null);
            $assinatura = "";
            openssl_sign($dados, $assinatura, $this->privateKeyId);
            $signatureValue = base64_encode($assinatura);


	        $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
	        $signature->appendChild($novaTag);

	        $keyInfo = $domXml->createElement("KeyInfo");
	        $signature->appendChild($keyInfo);

	        $X509Data = $domXml->createElement("X509Data");
	        $keyInfo->appendChild($X509Data);

	        $novaTag = $domXml->createElement("X509Certificate", $this->publicKey);
	        $X509Data->appendChild($novaTag);
    	}
        return $domXml;
	}

	private function getPublicKey($key) {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                    strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

    private function gerarXmlConsultarLoteRps($inscricaoPrestador, $inscricaoMunicipalPrestador, $protocolo) {
        $this->xmlConsultarLoteRps = '<ConsultarLoteRpsEnvio xmlns="http://www.abrasf.org.br/ABRASF/arquivos/nfse.xsd">
            <Prestador>
                <Cnpj>'.$inscricaoPrestador.'</Cnpj>
                <InscricaoMunicipal>'.$inscricaoMunicipalPrestador.'</InscricaoMunicipal>
            </Prestador>
            <Protocolo>'.$protocolo.'</Protocolo>
        </ConsultarLoteRpsEnvio>';
        return $this->xmlConsultarLoteRps;
    }

     public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();
        

        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];

        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

        $dadosNfct = Util::getDadosNfCt($this->connection, $notafiscal["CDFILIAL"], $notafiscal["NRLANCTONF"]);
        
        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);

        $this->ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
    
        $this->NumeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
        $this->IdLote = "LOTE_".$notafiscal["CDEMPRESA"]."_".$this->NumeroLote;
        $this->InscricaoPrestador = str_replace(array(".","-","/"), "", $notafiscal["NRINSJURIEST"]);
        $this->InscricaoMunicipalPrestador = str_replace(array(".","-","/"), "", $notafiscal["CDINSCMUNIEMP"]);
        
        $this->RazaoSocialPrestador = $notafiscal["NMRAZSOCEMIT"];

        if(!isset($dadosNfct[0])) {
            if($notafiscal["IDNATUOPEREMPR"] == 1) {
                $codigoMunicipoPrestador = (int)$notafiscal["CDMUNICIBGEEMP"];
            }
            else {
                $codigoMunicipoPrestador = (int)$notafiscal["CDMUNICIBGESERVPRES"];
            }
        }
        else {
            $codigoMunicipoPrestador = $dadosNfct[0]["CDMUNICIBGE"];
        }
        
        $this->CodigoMunicipioPrestador = $codigoMunicipoPrestador;

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            //VALIDACAO
            if (empty($notafiscal["CDINSCMUNIEMP"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDNATUOPEREMPR"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Natureza da operação deve ser informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDREGESPTRIBEMP"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Regime especial de tributação deve ser informado."
                );
            }
            if(!isset($dadosNfct[0])) {
                $naturezaOperacao = (int)$notafiscal["IDNATUOPEREMPR"];
            }
            else {
                if($dadosNfct[0]["CDMUNICIBGE"] == '3106200') {
                    $naturezaOperacao =  1;
                }
                else {
                    $naturezaOperacao =  2;
                }
            }
            $this->NaturezaOperacao = $naturezaOperacao;
            $this->RegimeEspecialTributacao = "";
            $this->OptanteSimplesNacional = 2;

            if ($notafiscal["IDOPTSIMPLES"] == "S") {
                $this->RegimeEspecialTributacao = (int)$notafiscal["IDREGESPTRIBEMP"];
                $this->OptanteSimplesNacional = 1;
            }
            $this->IncentivadorCultural = 2;

            $dataAtual = date_create();
            //$this->DataEmissao = $dataAtual->format("Y-m-d\TH:i:s");
            $this->DataEmissao = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");

            $this->NumeroRps = Util::inserirNumeroRps($this->connection, $this->NumeroLote, 2, $notafiscal["CDEMPRESA"], $notafiscal["CDSERIERPS"], null, $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

            $this->SerieRps = $notafiscal["CDSERIERPS"];
            $this->IdRps = "RPS_" . $this->SerieRps . "_" . $this->NumeroRps;
            $this->TipoRps = 1;
            $this->StatusRps = 1;

            $this->InscricaoTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSJURCLIE"]);
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA") {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];
            $this->Endereco = substr($notafiscal["DSENDECLIE"], 0, 125);
            $this->Numero = ".";
            $this->Complemento = substr($notafiscal["DSCOMPENDCLI"], 0, 60);
            $this->Bairro = substr($notafiscal["NMBAIRCLIE"], 0, 60);
            $this->CodigoMunicipioTomador = $notafiscal["CDMUNICIBGECLIE"];
            $this->Uf = $notafiscal["SGESTADOCLIE"];
            $this->Cep = substr(str_replace(array(".","-"), "", $notafiscal["NRCEPCLIE"]), 0, 8);
            $this->Telefone = substr(str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]), 0, 11);
            $this->Email = substr($notafiscal["DSEMAILCLIENFE"], 0, 80);

            $this->ValorServicos = $notafiscal["VRPRODUTOS"];
            $valorLiquido = $notafiscal["VRPRODUTOS"];
            $this->ValorDeducoes = 0.0;
            $this->ValorPis = 0.0;
            $this->ValorCofins = 0.0;
            $this->ValorInss = 0.0;
            $this->ValorIr = 0.0;
            $this->ValorCsll = 0.0;
            $this->IssRetido = 2;
            $this->ValorIss = 0.0;
            $this->ValorIssRetido = 0.0;
            $this->OutrasRetencoes = 0.0;
            $this->DescontoCondicionado = 0.0;
            $this->DescontoIncondicionado = 0.0;
            $this->BaseCalculo = 0.0;

            $impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            foreach ($impostos as $imp) {
                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorPis = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorPis;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCofins = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCofins;
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorInss = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorInss;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("22", "25")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorIr = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorIr;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCsll = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCsll;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("31", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->IssRetido = 1;
                        $this->ValorIssRetido = (float)$imp["VALORIMPOSTO"];
                        $this->ValorIss       += (float)$imp["VALORIMPOSTO"];
                        $valorLiquido -= $this->ValorIssRetido;
                    }
                    else {
                        $this->ValorIss += (float)$imp["VALORIMPOSTO"];
                        $this->BaseCalculo = $imp["BASECALCULO"];
                    }
                }
            }

            $outrasRetencoes = Util::getOutrasRetencoes($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            if (!empty($outrasRetencoes["VRIMPOSTOIT"]) && $outrasRetencoes["VRIMPOSTOIT"] > 0) {
                $this->OutrasRetencoes = $outrasRetencoes["VALORIMPOSTO"];
                $valorLiquido -= $this->OutrasRetencoes;
            }

            $this->ValorLiquidoNfse = $valorLiquido;

            if (count($impostos) > 0) {
                $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
                $this->Aliquota = 0;
                if (!empty($impostoItem)) {
                    $this->Aliquota = (float)$impostoItem["VRALIQUOTA"];
                }
            }
            else {
                //VALIDACAO
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }

            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $cdarvprod = $itemProduto[0]["CDARVPROD"];

            $discriminacaoNota = "";

            foreach ($itemProduto as $linha) {
                $discriminacaoNota .= $linha["DSPRODEMIS"] . "\r\n";
            }
            

            $itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            //$descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                //$descricaoEncargo .= $item["DESCRICAO"] . "\n";
                if ($item["IDENCARGO"] == "D") {
                    $this->DescontoIncondicionado += (float)$item["VRITEMENCA"];
                }
            }

            /*$descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }

            $this->Discriminacao = $descricaoProd;
            if (!empty($descricaoEncargo)) {
                $this->Discriminacao .= "|" . $descricaoEncargo;
            }
            if (!empty($descricaoDadosAdic)) {
                $this->Discriminacao .= "|" . $descricaoDadosAdic;
            }*/

            $this->Discriminacao = trim($discriminacaoNota);
            
            //$this->Discriminacao = substr(Util::retiraAcento($this->Discriminacao), 0, 2000);
            $this->Discriminacao = substr($this->Discriminacao, 0, 2000);
            $this->CodigoCnae = str_replace(array("-", "/", ".", " "), "", $notafiscal["CDCNAE"]);

            $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
            //VALIDACAO
            if (!is_array($codTribMuni) || empty($codTribMuni["CDCLASSERVPADR"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Código da classificação do serviço deve ser informado."
                );
            }
            else {
                $this->ItemListaServico = $codTribMuni["CDCLASSERVPADR"];//1.01
                $this->CodigoTributacaoMunicipio = $codTribMuni["CDTRIBPRODPADR"];
            }

            $this->adicionarRpsEnviarLote();

            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRps,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $this->NumeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }

        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $xml = $this->getXmlAssinadoEnviarLoteRps();
            return $this->enviarLoteRpsWS($xml, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $this->NumeroLote);
        }
    }

}